import requests
import urllib
from bs4 import BeautifulSoup
import re
from datetime import timedelta
import requests_cache
from datetime import date
from ratelimit import limits, sleep_and_retry


requests_cache.install_cache(
    cache_name='requests_cache', expire_after=timedelta(days=14))


@sleep_and_retry
@limits(calls=50,
        period=1)  # Allows 50 requets within 1 second; sleep till the request period has ellapsed and then retry
def find_mr_and_metadata(author, title,suffix):

    first_page, last_page = get_page_range_from_suffix(suffix)
     
    year = get_publication_year_from_suffix(suffix)
    payload = {
        "au": f"{author}",
        "ti": f"{title}",
        "jrnl": "",
        "ipage": f"{first_page}",
        "fpage": f"{last_page}",
        "year": f"{year}",
        "format": "mrcit",
        "Search": "Search",
    }

    html_text = requests.post(
        "https://mathscinet.ams.org/mrlookup", data=payload).text
    
    soup = BeautifulSoup(html_text, features="lxml")
    book = soup.find_all("tr")[13]
    
    if book is not None:
        # author = book.find("strong", class_="author").text
        # title = book.find("h3", class_="title").find("strong").text
        # suffix = book.find("div", class_="source").text.strip()
        mr = book.find("a").text
        return  "", "", "", mr
    return "", "", "", ""


def get_publication_year_from_suffix(suffix):
    start_year = 1700
    end_year = date.today().year + 1
    match = re.search("[^-–0-9]?([0-9]{4})[^-–0-9]?", suffix)
    if match is None:
        return ""
    year = int(match.group(1))
    if year >= start_year and year <= end_year:
        return year
    return ""

def get_page_range_from_suffix(suffix):
    match = re.search("\d{1,}[–-]\d{1,}",suffix)
    if match is None: 
        return "",""
    first_page, last_page = re.split("–|-",match[0])
    if first_page is not None and last_page is not None:
        return first_page, last_page
    return "",""
