import requests
import urllib
from bs4 import BeautifulSoup
import re
import requests_cache
from datetime import timedelta
from datetime import date
from ratelimit import limits, sleep_and_retry


requests_cache.install_cache(
    cache_name='requests_cache', expire_after=timedelta(days=14))


@sleep_and_retry
@limits(calls=50,
        period=1)  # Allows 50 requets within 1 second; sleep till the request period has ellapsed and then retry
def find_zbl_and_metadata(author, title, suffix):

    year = get_publication_year_from_suffix(suffix)
    query = urllib.parse.quote(f"au:({author}) & ti:{title} & py:{year}")

    html_text = requests.get(f'https://zbmath.org/?q={query}&r=relevance').text
    soup = BeautifulSoup(html_text,features="lxml")
    
    books = soup.find("div",class_= "list")
    book = soup.find("div",class_= "item")
  
    if books is not None:
        author = books.find("div",class_="author").text
        title = books.find("h3",class_="title").find("strong").text
        suffix = books.find("div",class_="source").text.strip()
        zbl = books.find("a",class_="label").text
        return author,title,suffix, zbl

    if book is not None:
        author = book.find("div",class_="author").text
        title = book.find("h2",class_="title").find("strong").text
        suffix = book.find("div",class_="source").text
        zbl = book.find("h2",class_="title").find("a",class_="label").text
        return author,title,suffix, zbl
    
    return "","","",""

def get_publication_year_from_suffix(suffix):
    start_year = 1700 
    end_year = date.today().year + 1 
    match = re.search("[^-–0-9]?([0-9]{4})[^-–0-9]?",suffix)
    if match is None:
        return ""
    year = int(match.group(1))
    if year >= start_year and year <= end_year:
        return year 
    return ""
    